local _, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:RegisterModule("Plus")

local pairs, type = pairs, type

local defaultSettings = {
	["debug"] = false,
	["Loot"] = true,
	["LootAnnounce"] = true,
	["LootAnnounceTitle"] = true,
	["LootAnnounceRarity"] = 1,
	["normTex"] = false,
	["Chat"] = true,
	["ChatEmote"] = false,
	["ChatClassColor"] = true,
	["ChatRaidIndex"] = false,
	["GlobalFade"] = false,
	["GlobalFadeAlpha"] = .1,
	["GlobalFadeDuration"] = .3,
	["GlobalFade_Combat"] = true,
	["GlobalFade_Target"] = true,
	["GlobalFade_Cast"] = true,
	["GlobalFade_Health"] = true,
	["GlobalFade_ActionBar"] = true,
	["GlobalFade_UnitFrame"] = false,
	["InboxMailBag"] = true,
	["TinyInspect"] = true,
	["ButtonForge"] = true,
	["Ace3"] = true,
	["ls_Toasts"] = true,
	["WhisperPop"] = true,
	["Immersion"] = true,
	["MeetingStone"] = true,
	["tdBattlePetScript"] = true,
}

local loader = CreateFrame("Frame")
loader:RegisterEvent("ADDON_LOADED")
loader:SetScript("OnEvent", function(self, _, addon)
	if addon ~= "NDui_Plus" then return end

	for key in pairs(NDuiPlusDB) do
		if defaultSettings[key] == nil then
			NDuiPlusDB[key] = nil
		end
	end

	for key, value in pairs(defaultSettings) do
		if NDuiPlusDB[key] == nil then
			if type(value) == "table" then
				NDuiPlusDB[key] = {}
				for k in pairs(value) do
					NDuiPlusDB[key][k] = value[k]
				end
			else
				NDuiPlusDB[key] = value
			end
		end
	end

	if NDuiPlusDB["normTex"] then DB.normTex = P.normTex end

	self:UnregisterAllEvents()
end)

function P:Debug(...)
	if NDuiPlusDB["debug"] then
		_G.DEFAULT_CHAT_FRAME:AddMessage("|cFF70B8FFNDui_Plus:|r " .. format(...))
	end
end

function P:Print(...)
	_G.DEFAULT_CHAT_FRAME:AddMessage("|cFF70B8FFNDui_Plus:|r " .. format(...))
end

function P:VersionCheck_Compare(new, old)
	local new1, new2, new3 = strsplit(".", new)
	new1, new2, new3 = tonumber(new1), tonumber(new2), tonumber(new3)

	local old1, old2, old3 = strsplit(".", old)
	old1, old2, old3 = tonumber(old1), tonumber(old2), tonumber(old3)

	if new1 > old1 or (new1 == old1 and new2 > old2) or (new1 == old1 and new2 == old2 and new3 > old3) then
		return "IsNew"
	elseif new1 < old1 or (new1 == old1 and new2 < old2) or (new1 == old1 and new2 == old2 and new3 < old3) then
		return "IsOld"
	end
end

function P:OnLogin()
	local status = self:VersionCheck_Compare(DB.Version, P.SupportVersion)
	if status == "IsOld" then
		self:Print(format(L["Version Check"], P.SupportVersion))
		return
	end

	self:Chat()
	self:Skins()
	self:LootEx()
	self:Misc()
	self:GlobalFade()
end
