local _, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:GetModule("Plus")
local Bar = B:GetModule("Actionbar")
-----------------
-- Credit: ElvUI
-----------------
P.GuidCache = {}
P.ClassNames = {}
P.GroupNames = {}

P.ChatEvents = {
	"CHAT_MSG_BATTLEGROUND",
	"CHAT_MSG_BATTLEGROUND_LEADER",
	"CHAT_MSG_BN_WHISPER",
	"CHAT_MSG_BN_WHISPER_INFORM",
	"CHAT_MSG_CHANNEL",
	"CHAT_MSG_GUILD",
	"CHAT_MSG_INSTANCE_CHAT",
	"CHAT_MSG_INSTANCE_CHAT_LEADER",
	"CHAT_MSG_OFFICER",
	"CHAT_MSG_PARTY",
	"CHAT_MSG_PARTY_LEADER",
	"CHAT_MSG_RAID",
	"CHAT_MSG_RAID_LEADER",
	"CHAT_MSG_RAID_WARNING",
	"CHAT_MSG_SAY",
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_WHISPER",
	"CHAT_MSG_WHISPER_INFORM",
	"CHAT_MSG_YELL",
}

local isCalling = false
function P:GetPlayerInfoByGUID(guid)
	if isCalling then return end

	local data = P.GuidCache[guid]
	if not data then
		isCalling = true
		local ok, localizedClass, englishClass, localizedRace, englishRace, sex, name, realm = pcall(GetPlayerInfoByGUID, guid)
		isCalling = false

		if not (ok and englishClass) then return end

		if realm == '' then realm = nil end
		local nameWithRealm
		if name and name ~= '' then
			nameWithRealm = (realm and name..'-'..realm) or name..'-'..DB.MyRealm
		end

		data = {
			localizedClass = localizedClass,
			englishClass = englishClass,
			localizedRace = localizedRace,
			englishRace = englishRace,
			sex = sex,
			name = name,
			realm = realm,
			nameWithRealm = nameWithRealm,
		}

		if name then
			P.ClassNames[strlower(name)] = englishClass
		end
		if nameWithRealm then
			P.ClassNames[strlower(nameWithRealm)] = englishClass
		end

		P.GuidCache[guid] = data
	end

	return data
end

function P:ClassFilter(message)
	local isFirstWord, rebuiltString
	for word in gmatch(message, '%s-%S+%s*') do
		local tempWord = gsub(word,'^[%s%p]-([^%s%p]+)([%-]?[^%s%p]-)[%s%p]*$','%1%2')
		local lowerCaseWord = strlower(tempWord)

		local classMatch = P.ClassNames[lowerCaseWord]
		local wordMatch = classMatch and lowerCaseWord

		if wordMatch then
			local r, g, b = B.ClassColor(classMatch)
			word = gsub(word, gsub(tempWord, '%-','%%-'), format('\124cff%.2x%.2x%.2x%s\124r', r*255, g*255, b*255, tempWord))
		end

		if not isFirstWord then
			rebuiltString = word
			isFirstWord = true
		else
			rebuiltString = format('%s%s', rebuiltString, word)
		end
	end

	return rebuiltString
end

function P:UpdateBubbleColor()
	if not NDuiPlusDB["ChatClassColor"] then return end

	local backdrop = self.backdrop
	local str = backdrop and backdrop.String
	local text = str and str:GetText()
	local rebuiltString = text and P:ClassFilter(text)

	if rebuiltString then
		str:SetText(RemoveExtraSpaces(rebuiltString))
	end
end

function P:HookBubble(frame, backdrop)
	if frame.isHooked then return end

	if not frame.backdrop then
		frame.backdrop = backdrop
		frame:HookScript('OnShow', P.UpdateBubbleColor)
		P.UpdateBubbleColor(frame)
	end

	frame.isHooked = true
end

function P:UpdateChatColor(event, msg, ...)
	msg = P:ClassFilter(msg) or msg
	return false, msg, ...
end

function P:ChatClassColor()
	if not NDuiPlusDB["ChatClassColor"] then return end

	for _, event in pairs(P.ChatEvents) do
		ChatFrame_AddMessageEventFilter(event, P.UpdateChatColor)
	end

	hooksecurefunc("GetPlayerInfoByGUID",function(...) 
		P:GetPlayerInfoByGUID(...)
	end)
end
------------------------
-- Credit: BasicChatMods
------------------------
function P:UpdateGroupNames()
	if not IsInRaid() then
		wipe(P.GroupNames)
		return
	end

	for i = 1, GetNumGroupMembers() do
		local name, _, subgroup = GetRaidRosterInfo(i)
		if name and subgroup then
			P.GroupNames[name] = tostring(subgroup)
		end
	end
end

local function addRaidIndex(fullName, nameString, nameText)
	local name = Ambiguate(fullName, "none")
	local group = name and P.GroupNames[name]

	if group then
		nameText = nameText..":"..group
	end

	return "|Hplayer:"..fullName..nameString.."["..nameText.."]|h"
end

function P:UpdateRaidIndex(text, ...)
	if IsInRaid() and NDuiPlusDB["ChatRaidIndex"] then
		text = text:gsub("|Hplayer:([^:|]+)([^%[]+)%[([^%]]+)%]|h", addRaidIndex)
	end

	return self.origAddMsg(self, text, ...)
end

function P:ChatRaidIndex()
	local eventList = {
		"GROUP_ROSTER_UPDATE",
		"PLAYER_ENTERING_WORLD",
	}

	for _, event in next, eventList do
		B:RegisterEvent(event, P.UpdateGroupNames)
	end

	for i = 1, NUM_CHAT_WINDOWS do
		if i ~= 2 then
			local chatFrame = _G["ChatFrame"..i]
			chatFrame.origAddMsg = chatFrame.AddMessage
			chatFrame.AddMessage = P.UpdateRaidIndex
		end
	end
end

function P:Chat()
	if not NDuiPlusDB["Chat"] then return end

	self:ChatEmote()
	self:ChatClassColor()
	self:ChatRaidIndex()
end